/*
Copyright 2020 The Flux authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package main

import (
	"github.com/spf13/cobra"

	sourcev1 "github.com/fluxcd/source-controller/api/v1"
)

var reconcileSourceHelmChartCmd = &cobra.Command{
	Use:   "chart [name]",
	Short: "Reconcile a HelmChart source",
	Long:  `The reconcile source command triggers a reconciliation of a HelmChart resource and waits for it to finish.`,
	Example: `  # Trigger a reconciliation for an existing source
  flux reconcile source chart podinfo
 
  # Trigger a reconciliation of the HelmCharts's source and apply changes
  flux reconcile helmchart podinfo --with-source`,
	ValidArgsFunction: resourceNamesCompletionFunc(sourcev1.GroupVersion.WithKind(sourcev1.HelmChartKind)),
	RunE: reconcileWithSourceCommand{
		apiType: helmChartType,
		object:  helmChartAdapter{&sourcev1.HelmChart{}},
	}.run,
}

func init() {
	reconcileSourceHelmChartCmd.Flags().BoolVar(&rhcArgs.syncHrWithSource, "with-source", false, "reconcile HelmChart source")
	reconcileSourceCmd.AddCommand(reconcileSourceHelmChartCmd)
}

func (obj helmChartAdapter) lastHandledReconcileRequest() string {
	return obj.Status.GetLastHandledReconcileRequest()
}

type reconcileHelmChartFlags struct {
	syncHrWithSource bool
}

var rhcArgs reconcileHelmChartFlags

func (obj helmChartAdapter) reconcileSource() bool {
	return rhcArgs.syncHrWithSource
}

func (obj helmChartAdapter) getSource() (reconcileSource, sourceReference) {
	var cmd reconcileSource
	switch obj.Spec.SourceRef.Kind {
	case sourcev1.HelmRepositoryKind:
		cmd = reconcileCommand{
			apiType: helmRepositoryType,
			object:  helmRepositoryAdapter{&sourcev1.HelmRepository{}},
		}
	case sourcev1.GitRepositoryKind:
		cmd = reconcileCommand{
			apiType: gitRepositoryType,
			object:  gitRepositoryAdapter{&sourcev1.GitRepository{}},
		}
	case sourcev1.BucketKind:
		cmd = reconcileCommand{
			apiType: bucketType,
			object:  bucketAdapter{&sourcev1.Bucket{}},
		}
	}

	return cmd, sourceReference{
		kind:      obj.Spec.SourceRef.Kind,
		name:      obj.Spec.SourceRef.Name,
		namespace: obj.Namespace,
	}
}

func (obj helmChartAdapter) isStatic() bool {
	return false
}
